import { ResponseError, EdmSchema, ClientDataService, ClientDataContext } from '@themost/client';
import * as jQuery from 'jquery';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var DateTimeRegex = /^\d{4}-([0]\d|1[0-2])-([0-2]\d|3[01])(?:[T ](\d+):(\d+)(?::(\d+)(?:\.(\d+))?)?)?(?:Z(-?\d*))?$/g;
// eslint-disable-next-line prefer-arrow/prefer-arrow-functions
function jsonReviver(key, value) {
    if (DateTimeRegex.test(value)) {
        return new Date(value);
    }
    return value;
}
var JQueryDataService = /** @class */ (function (_super) {
    __extends(JQueryDataService, _super);
    function JQueryDataService(base, options) {
        return _super.call(this, base, options) || this;
    }
    JQueryDataService.prototype.execute = function (options) {
        var _this = this;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        var headers = __assign(__assign(__assign({}, this.getHeaders()), {
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        }), options.headers);
        var reviver = this.getOptions().useJsonReviver || jsonReviver;
        return new Promise(function (resolve, reject) {
            void jQuery.ajax({
                url: _this.resolve(options.url),
                headers: headers,
                method: options.method,
                crossDomain: true,
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                data: options.data,
                dataType: 'text',
                xhrFields: {
                    withCredentials: true
                }
            }).done(function (data, textStatus, jqXHR) {
                if (jqXHR.status === 204) {
                    return resolve(null);
                }
                else {
                    // safely handle empty body
                    if ((typeof data === 'string' && data.length === 0)) {
                        return resolve(null);
                    }
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    return resolve(JSON.parse(data, reviver));
                }
            }).fail(function (jqXHR, textStatus, errorThrown) {
                return reject(new ResponseError(errorThrown, jqXHR.status));
            });
        });
    };
    JQueryDataService.prototype.getMetadata = function () {
        var _this = this;
        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
        var headers = __assign(__assign({}, this.getHeaders()), {
            'Accept': 'application/xml'
        });
        return new Promise(function (resolve, reject) {
            void jQuery.get({
                url: _this.resolve('$metadata'),
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                headers: headers,
                crossDomain: true,
                dataType: 'text',
                xhrFields: {
                    withCredentials: true
                }
            }).done(function (data, textStatus, jqXHR) {
                if (jqXHR.status === 204) {
                    return resolve(null);
                }
                else {
                    // safely handle empty body
                    if ((typeof data === 'string' && data.length === 0)) {
                        return resolve(null);
                    }
                    // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                    return resolve(EdmSchema.loadXML(data));
                }
            }).fail(function (jqXHR, textStatus, errorThrown) {
                return reject(new ResponseError(errorThrown, jqXHR.status));
            });
        });
    };
    return JQueryDataService;
}(ClientDataService));

var JQueryDataContext = /** @class */ (function (_super) {
    __extends(JQueryDataContext, _super);
    function JQueryDataContext(config) {
        return _super.call(this, new JQueryDataService(config.base, config.options), config.options) || this;
    }
    return JQueryDataContext;
}(ClientDataContext));
// noinspection JSUnusedGlobalSymbols
jQuery.fn.extend({
    dataContext: function dataContext(config) {
        // eslint-disable-next-line @typescript-eslint/no-this-alias,@typescript-eslint/no-unsafe-assignment
        var element = this;
        if (element != null) {
            if (config === 'getContext') {
                return element.data('JQueryDataContext');
            }
            var context = element.data('JQueryDataContext');
            if (context == null) {
                context = new JQueryDataContext(config);
            }
            element.data('JQueryDataContext', context);
        }
        return element;
    }
});

export { JQueryDataContext, JQueryDataService };
//# sourceMappingURL=index.esm.js.map
